﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;

namespace howto_redeye_reduction2
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        // Open a file.
        Bitmap TheBitmap;
        private void mnuFileOpen_Click(object sender, EventArgs e)
        {
            if (ofdImage.ShowDialog() == DialogResult.OK)
            {
                TheBitmap = new Bitmap(ofdImage.FileName);
                picImage.Image = TheBitmap;
                this.ClientSize = new Size(
                    picImage.Right + picImage.Left,
                    picImage.Bottom + picImage.Left);
                picImage.Visible = true;
            }
        }

        // Save the file.
        private void mnuFileSaveAs_Click(object sender, EventArgs e)
        {
            if (sfdImage.ShowDialog() == DialogResult.OK)
            {
                SaveBitmapUsingExtension(TheBitmap, sfdImage.FileName);
            }
        }

        // Save the file with the appropriate format.
        // Throw a NotSupportedException if the file
        // has an unknown extension.
        public void SaveBitmapUsingExtension(Bitmap bm, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    bm.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    bm.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    bm.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    bm.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    bm.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    bm.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }

        // Perform red-eye reduction starting with the clicked pixel.
        private void picImage_MouseClick(object sender, MouseEventArgs e)
        {
            Bitmap bm = (Bitmap)picImage.Image;
            RemoveRedEyeAtPoint(1, bm, e.X, e.Y);
            picImage.Image = bm;
        }

        // Remove red-eye in the rectangle.
        private void RemoveRedEyeAtPoint(int depth, Bitmap bm, int x, int y)
        {
            // Don't recurse too deeply.
            if (depth > 100) return;

            // Do nothing if this pixel isn't mostly red.
            Color clr = bm.GetPixel(x, y);
            if ((clr.R <= clr.G + 10) || (clr.R <= clr.B + 10)) return;
                
            // Remove red-eye at this point.
            byte new_clr = (byte)((clr.R + clr.G + clr.B) / 3);
            bm.SetPixel(x, y, Color.FromArgb(new_clr, new_clr, new_clr));

            // Check adjacent pixels.
            int min_x = Math.Max(x - 1, 0);
            int max_x = Math.Min(x + 1, bm.Width - 1);
            int min_y = Math.Max(y - 1, 0);
            int max_y = Math.Min(y + 1, bm.Height - 1);
            for (int new_x = min_x; new_x <= max_x; new_x++)
            {
                for (int new_y = min_y; new_y <= max_y; new_y++)
                {
                    if ((new_x != 0) || (new_y != 0))
                        RemoveRedEyeAtPoint(depth + 1, bm, new_x, new_y);
                }
            }
        }
    }
}
